/******************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 	*
 *  STM3210E-EVAL:  PROJEKT_15 silnik krokowy  				*
 *														 sterowanie napieciowe	*
 ******************************************************/
#include "stm32f10x.h"
#include "stm3210e_eval_lcd.h"
#include "foto.h"
#include <stdio.h>
#define Brown 0xEBC2
#define Amber 0xFDEA

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
int SysTick_Config_Div8(unsigned int Ticks);
void Delay(unsigned int);

// zmienne globalne, SM = Stepper Motor -----------------------
unsigned char txt2LCD[20] = {0}; 		// bufor funkcji sprintf
unsigned int SMcounter = 0, SMclockwise = 1;

int main(void) {
  unsigned int i;
	unsigned int load[10];
	
	// wartosci rejestru SysTick -> LOAD (predkosci obrotowych) -
	load[0] = 2700000;
	load[1] = 1350000; 	//   1 obrot / minute
	load[2] =  675000; 	//   2 
	load[3] =  270000; 	//   5 
	load[4] =  135000; 	//  10 
	load[5] =   67500; 	//  20 
	load[6] =   33750; 	//  40 
	load[7] =   22500; 	//  60
	load[8] =   16875; 	//  80 
	load[9] =   13500; 	// 100
	
  //konfiguracja systemu --------------------------------------
  RCC_Config();		
  GPIO_Config();
	NVIC_Config();
	SysTick_Config_Div8(9000000ul);
	STM3210E_LCD_Init();		// inicjalizacja TFT LCD 240320L8
  GPIO_ResetBits(GPIOF, GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9);	// zgasimy wszystkie diody LED
	LCD_Clear(Black);		// Clear the LCD
	LCD_WriteBMP((uint32_t)clockwise);
  LCD_SetBackColor(Black);		// kolor tla
	LCD_SetTextColor(Red);   // kolor textu
	LCD_SetFont(&Font16x24);
	
  while (1) {
		for (i=0; i<10; i++){
			SysTick->LOAD = (load[i] & SysTick_LOAD_RELOAD_Msk) - 1;
			sprintf((char *)txt2LCD,"  obroty = %3i RPM",1350000/load[i]);
			LCD_DisplayStringLine(Line9, (uint8_t *)txt2LCD);
			Delay(5*8000000);
		}
		for (i=0; i<10; i++){
			SysTick->LOAD = (load[9-i] & SysTick_LOAD_RELOAD_Msk) - 1;
			sprintf((char *)txt2LCD,"  obroty = %3i RPM",1350000/load[9-i]);
			LCD_DisplayStringLine(Line9, (uint8_t *)txt2LCD);
			Delay(5*8000000);
		}
		if(SMclockwise){
			SMclockwise = 0;
			LCD_Clear(Black);		// Clear the LCD
			LCD_WriteBMP((uint32_t)clockwise2);
			LCD_SetTextColor(Red);   // kolor textu
		}
		else{
			SMclockwise = 1;
			LCD_Clear(Black);		// Clear the LCD
			LCD_WriteBMP((uint32_t)clockwise);
			LCD_SetTextColor(Yellow);   // kolor textu
		}
  }
//  return 0;  // tego 'return' nigdy nie osiagnie 
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE);// wlacz taktowanie portu GPIO F
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  /*Tu nalezy umiescic kod zwiazany z konfiguracja poszczegolnych portow GPIO potrzebnych w programie*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOF, &GPIO_InitStructure);
	
}
// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}

void Delay(unsigned int counter){
	// opoznienie programowe, 8'000'000 to jedna sekunda
	while (counter--){
		__NOP();
		__NOP();
	}
}
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/




