/**********************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 			*
 *  STM3210E-EVAL:  PROJEKT_19 szescian, Fixed Point			*
 **********************************************************/
#include "stm32f10x.h"
#include "stm3210e_eval_lcd.h"
#include "AsmQ32.h"
#include <stdio.h>
#include <math.h>
#define Brown 0xEBC2
#define Amber 0xFDEA
const double pi = 3.14159265358979323846;

// zmienne globalne -------------------------------------------
uint8_t str2disp[] = {"                    \0"};
const int N = sizeof(str2disp) - 1;

// prototypy funkcji ------------------------------------------
int  RCC_Config(void);
void GPIO_Config(void);
void Delay(unsigned int);
void MatrixMux(int[3], int[3][3]);

// liczby -----------------------
//         60.0 --> 0x003C0000
//        -60.0 --> 0xFFC40000
//          1.0 --> 0x00010000
//  sin(alfa_x) --> 0x00000478	alfa_x = pi/180.0; (jeden stopien)
// -sin(alfa_x) --> 0xFFFFFB88
//  cos(alfa_x) --> 0x0000FFF6
//  sin(alfa_y) --> 0xFFFFF29A	alfa_y = pi/-60.0; (minus trzy stopnie)
// -sin(alfa_y) --> 0x00000D66
//  cos(alfa_y) --> 0x0000FFA6
//  sin(alfa_z) --> 0x000008EF	alfa_z = pi/90.0;	(dwa stopnie)
// -sin(alfa_z) --> 0xFFFFF711
//  cos(alfa_z) --> 0x0000FFD8

// main -------------------------------------------------------
int main(void) {
  unsigned int i, j;
	int P00[3]={0xFFC40000, 0xFFC40000, 0xFFC40000};		// baza
	int P01[3]={0xFFC40000, 0x003C0000, 0xFFC40000};
	int P02[3]={0x003C0000, 0x003C0000, 0xFFC40000};
	int P03[3]={0x003C0000, 0xFFC40000, 0xFFC40000};
	int P10[3]={0xFFC40000, 0xFFC40000, 0x003C0000};		// baza
	int P11[3]={0xFFC40000, 0x003C0000, 0x003C0000};
	int P12[3]={0x003C0000, 0x003C0000, 0x003C0000};
	int P13[3]={0x003C0000, 0xFFC40000, 0x003C0000};
	int P00new[3], P01new[3], P02new[3], P03new[3];	// nowe
	int P10new[3], P11new[3], P12new[3], P13new[3];	// nowe
	int P00old[3], P01old[3], P02old[3], P03old[3];	// stare
	int P10old[3], P11old[3], P12old[3], P13old[3];	// stare
	// macierze obrotu
	int Ax[3][3], Ay[3][3], Az[3][3];

  //konfiguracja systemu --------------------------------------
  RCC_Config();		
  GPIO_Config(); 
  GPIO_ResetBits(GPIOF, GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9);	// zgasimy wszystkie diody LED
  STM3210E_LCD_Init();	// inicjalizacja TFT LCD 240320L8
	LCD_Clear(Black);			// Clear the LCD
	
	// napis ----------------------------------------------------
  LCD_SetBackColor(Black);		// kolor tla
  LCD_SetTextColor(Amber);		// kolor textu
	LCD_SetFont(&Font16x24);
  LCD_DisplayStringLine(Line9, (uint8_t *)"Projekt19_FixedPoint");
	
	// tworzymy macierze obrotu ---------------------------------
	for(i=0; i<3; i++){
		for(j=0; j<3; j++){
			Ax[i][j] = 0;
			Ay[i][j] = 0;
			Az[i][j] = 0;
		}
	}
	Ax[0][0] = 0x00010000; 
	Ax[1][2] = 0x00000478; Ax[2][1] = 0xFFFFFB88;
	Ax[1][1] = 0x0000FFF6; Ax[2][2] = 0x0000FFF6;
	Ay[1][1] = 0x00010000; 
	Ay[2][0] = 0xFFFFF29A; Ay[0][2] = 0x00000D66;
	Ay[0][0] = 0x0000FFA6; Ay[2][2] = 0x0000FFA6;
	Az[2][2] = 0x00010000;
	Az[0][1] = 0x000008EF; Az[1][0] = 0xFFFFF711;
	Az[0][0] = 0x0000FFD8; Az[1][1] = 0x0000FFD8;
	
	// do wyswietlenia: baza + przesuniecie
	P00new[0] = Q1516_int(P00[0])+108; P00new[1] = Q1516_int(P00[1])+160;
	P01new[0] = Q1516_int(P01[0])+108; P01new[1] = Q1516_int(P01[1])+160;
	P02new[0] = Q1516_int(P02[0])+108; P02new[1] = Q1516_int(P02[1])+160;
	P03new[0] = Q1516_int(P03[0])+108; P03new[1] = Q1516_int(P03[1])+160;
	P10new[0] = Q1516_int(P10[0])+108; P10new[1] = Q1516_int(P10[1])+160;
	P11new[0] = Q1516_int(P11[0])+108; P11new[1] = Q1516_int(P11[1])+160;
	P12new[0] = Q1516_int(P12[0])+108; P12new[1] = Q1516_int(P12[1])+160;
	P13new[0] = Q1516_int(P13[0])+108; P13new[1] = Q1516_int(P13[1])+160;
	// obracamy kwadrat na ekranie ------------------------------
	while(1){
		for(i=0; i<3; i++){
			P00old[i] = P00new[i];
			P01old[i] = P01new[i];
			P02old[i] = P02new[i];
			P03old[i] = P03new[i];
			P10old[i] = P10new[i];
			P11old[i] = P11new[i];
			P12old[i] = P12new[i];
			P13old[i] = P13new[i];
		}
		// obrot = wylicze nowe wspolrzedne punktow bazy
		MatrixMux(P00, Az);		// os OZ
		MatrixMux(P01, Az);
		MatrixMux(P02, Az);
		MatrixMux(P03, Az);
		MatrixMux(P10, Az);
		MatrixMux(P11, Az);
		MatrixMux(P12, Az);
		MatrixMux(P13, Az);
		MatrixMux(P00, Ay);		// os OY
		MatrixMux(P01, Ay);
		MatrixMux(P02, Ay);
		MatrixMux(P03, Ay);
		MatrixMux(P10, Ay);
		MatrixMux(P11, Ay);
		MatrixMux(P12, Ay);
		MatrixMux(P13, Ay);
		MatrixMux(P00, Ax);		// os OX
		MatrixMux(P01, Ax);
		MatrixMux(P02, Ax);
		MatrixMux(P03, Ax);
		MatrixMux(P10, Ax);
		MatrixMux(P11, Ax);
		MatrixMux(P12, Ax);
		MatrixMux(P13, Ax);
		
		P00new[0] = Q1516_int(P00[0])+108; P00new[1] = Q1516_int(P00[1])+160;
		P01new[0] = Q1516_int(P01[0])+108; P01new[1] = Q1516_int(P01[1])+160;
		P02new[0] = Q1516_int(P02[0])+108; P02new[1] = Q1516_int(P02[1])+160;
		P03new[0] = Q1516_int(P03[0])+108; P03new[1] = Q1516_int(P03[1])+160;
		P10new[0] = Q1516_int(P10[0])+108; P10new[1] = Q1516_int(P10[1])+160;
		P11new[0] = Q1516_int(P11[0])+108; P11new[1] = Q1516_int(P11[1])+160;
		P12new[0] = Q1516_int(P12[0])+108; P12new[1] = Q1516_int(P12[1])+160;
		P13new[0] = Q1516_int(P13[0])+108; P13new[1] = Q1516_int(P13[1])+160;
		
		Delay(400000);
		
		LCD_SetTextColor(Black);		// kolor textu = wygaszam
		// pierwszy kwadrat
		LCD_DrawUniLine((uint16_t) P00old[0], (uint16_t) P00old[1], (uint16_t) P01old[0], (uint16_t) P01old[1]);
		LCD_DrawUniLine((uint16_t) P01old[0], (uint16_t) P01old[1], (uint16_t) P02old[0], (uint16_t) P02old[1]);
		LCD_DrawUniLine((uint16_t) P02old[0], (uint16_t) P02old[1], (uint16_t) P03old[0], (uint16_t) P03old[1]);
		LCD_DrawUniLine((uint16_t) P03old[0], (uint16_t) P03old[1], (uint16_t) P00old[0], (uint16_t) P00old[1]);
		// drugi kwadrat
		LCD_DrawUniLine((uint16_t) P10old[0], (uint16_t) P10old[1], (uint16_t) P11old[0], (uint16_t) P11old[1]);
		LCD_DrawUniLine((uint16_t) P11old[0], (uint16_t) P11old[1], (uint16_t) P12old[0], (uint16_t) P12old[1]);
		LCD_DrawUniLine((uint16_t) P12old[0], (uint16_t) P12old[1], (uint16_t) P13old[0], (uint16_t) P13old[1]);
		LCD_DrawUniLine((uint16_t) P13old[0], (uint16_t) P13old[1], (uint16_t) P10old[0], (uint16_t) P10old[1]);
		// linie pomiedzy kwadratami
		LCD_DrawUniLine((uint16_t) P00old[0], (uint16_t) P00old[1], (uint16_t) P10old[0], (uint16_t) P10old[1]);
		LCD_DrawUniLine((uint16_t) P01old[0], (uint16_t) P01old[1], (uint16_t) P11old[0], (uint16_t) P11old[1]);
		LCD_DrawUniLine((uint16_t) P02old[0], (uint16_t) P02old[1], (uint16_t) P12old[0], (uint16_t) P12old[1]);
		LCD_DrawUniLine((uint16_t) P03old[0], (uint16_t) P03old[1], (uint16_t) P13old[0], (uint16_t) P13old[1]);
	
		// wyswietlam
		LCD_SetTextColor(Yellow);		// kolor textu - wyswietlam
		LCD_DrawUniLine((uint16_t) P00new[0], (uint16_t) P00new[1], (uint16_t) P01new[0], (uint16_t) P01new[1]);
		LCD_DrawUniLine((uint16_t) P10new[0], (uint16_t) P10new[1], (uint16_t) P11new[0], (uint16_t) P11new[1]);
		LCD_SetTextColor(Red);
		LCD_DrawUniLine((uint16_t) P01new[0], (uint16_t) P01new[1], (uint16_t) P02new[0], (uint16_t) P02new[1]);
		LCD_DrawUniLine((uint16_t) P11new[0], (uint16_t) P11new[1], (uint16_t) P12new[0], (uint16_t) P12new[1]);
		LCD_SetTextColor(Blue);
		LCD_DrawUniLine((uint16_t) P02new[0], (uint16_t) P02new[1], (uint16_t) P03new[0], (uint16_t) P03new[1]);
		LCD_DrawUniLine((uint16_t) P12new[0], (uint16_t) P12new[1], (uint16_t) P13new[0], (uint16_t) P13new[1]);
		LCD_SetTextColor(Green);
		LCD_DrawUniLine((uint16_t) P03new[0], (uint16_t) P03new[1], (uint16_t) P00new[0], (uint16_t) P00new[1]);
		LCD_DrawUniLine((uint16_t) P13new[0], (uint16_t) P13new[1], (uint16_t) P10new[0], (uint16_t) P10new[1]);
		// linie pomiedzy kwadratami
		LCD_SetTextColor(White);
		LCD_DrawUniLine((uint16_t) P00new[0], (uint16_t) P00new[1], (uint16_t) P10new[0], (uint16_t) P10new[1]);
		LCD_DrawUniLine((uint16_t) P01new[0], (uint16_t) P01new[1], (uint16_t) P11new[0], (uint16_t) P11new[1]);
		LCD_SetTextColor(Magenta);
		LCD_DrawUniLine((uint16_t) P02new[0], (uint16_t) P02new[1], (uint16_t) P12new[0], (uint16_t) P12new[1]);
		LCD_DrawUniLine((uint16_t) P03new[0], (uint16_t) P03new[1], (uint16_t) P13new[0], (uint16_t) P13new[1]);
	}
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE);// wlacz taktowanie portu GPIO F
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  /*Tu nalezy umiescic kod zwiazany z konfiguracja poszczegolnych portow GPIO potrzebnych w programie*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOF, &GPIO_InitStructure);
	
}
//-------- mnozenie: wektor = macierz * wektor (x=A*x)---------
void MatrixMux(int x[3], int A[3][3]){
	int y[3] = {0, 0, 0};
	int x0, x1, x2, yi;
	x0 = Q1516_mult(x[0], A[0][0]); 
	x1 = Q1516_mult(x[1], A[0][1]);
	x2 = Q1516_mult(x[2], A[0][2]);
	yi = Q32_add(x0,x1); y[0] = Q32_add(yi,x2);
	
	x0 = Q1516_mult(x[0], A[1][0]); 
	x1 = Q1516_mult(x[1], A[1][1]);
	x2 = Q1516_mult(x[2], A[1][2]);
	yi = Q32_add(x0,x1); y[1] = Q32_add(yi,x2);
	
	x0 = Q1516_mult(x[0], A[2][0]); 
	x1 = Q1516_mult(x[1], A[2][1]);
	x2 = Q1516_mult(x[2], A[2][2]);
	yi = Q32_add(x0,x1); y[2] = Q32_add(yi,x2);
	
	x[0] = y[0]; x[1] = y[1]; x[2] = y[2]; 
}
//-------------------------------------------------------------
void Delay(unsigned int counter){
	// opoznienie programowe, 8'000'000 to jedna sekunda
	while (counter--){
		__NOP();
		__NOP();
	}
}

/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



