/******************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 	*
 *  STM3210E-EVAL:  PROJEKT_21 przetwarzanie ADC			*
 ******************************************************/
#include "stm32f10x.h"
#include "stm3210e_eval_lcd.h"
#include <stdio.h>
#define Brown 0xEBC2
#define Amber 0xFDEA

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
void ADC_Config(void);
int SysTick_Config_Div8(unsigned int Ticks);
void Delay(unsigned int);

// zmienne globalne, SM = Stepper Motor -----------------------
unsigned char txt2LCD[20] = {0}; 		// bufor funkcji sprintf
unsigned int ADC1_DR;			// wartosc binarna odczytana z ADC

int main(void) {
  //------------------- deklaracje zmiennych ------------------
  float voltage;
	unsigned int y;
	
  //konfiguracja systemu --------------------------------------
  RCC_Config();		
  GPIO_Config();
	NVIC_Config();
	ADC_Config();						// inicjalizacja ADC1
	STM3210E_LCD_Init();		// inicjalizacja TFT LCD 240320L8
  GPIO_ResetBits(GPIOF, GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9);	// zgasimy wszystkie diody LED
	LCD_Clear(Black);		// Clear the LCD
  LCD_SetBackColor(Black);		// kolor tla
	LCD_SetFont(&Font16x24);
	ADC_SoftwareStartConvCmd(ADC1, ENABLE); // rozpocznij przetwarzanie ADC
	while (!ADC_GetFlagStatus(ADC1,ADC_FLAG_EOC));	// czekamy na zakonczenie konwersji
	SysTick_Config_Div8(900000ul);	// przerwanie co 0.1 sekundy
	
	label:
  for(y=0; y<320; y++){
		voltage = ADC1_DR * 0.0008059;									// przeliczone na napiecie
		sprintf((char *)txt2LCD,"  U = %f [V]",voltage);
		LCD_SetTextColor(Amber);   // kolor textu
		LCD_DisplayStringLine(Line9, (uint8_t *)txt2LCD);
		
		LCD_SetTextColor(Black);   // kasujemy poprzedni punkt
		LCD_DrawLine(0, 319-y, 216, LCD_DIR_VERTICAL);
		LCD_SetTextColor(Green);   // kolor linii
		LCD_DrawLine(205-ADC1_DR/20, 319-y, 1, LCD_DIR_HORIZONTAL);

		Delay(1000000);

  }
	goto label;
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii
		RCC_ADCCLKConfig(RCC_PCLK2_Div6);                    // ADCCLK = PCLK2/6 = 72 MHz /6 = 12 MHz
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); // wlacz taktowanie ADC1
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE);// port GPIO F : LEDy
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);// port GPIO C : PC4 = potencjometr
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  /*Tu nalezy umiescic kod zwiazany z konfiguracja poszczegolnych portow GPIO potrzebnych w programie*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOF, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;   
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;	//	wejscie analogowe z potencjometru
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
}
// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}
// -------------- konfiguracja przetwornika ADC1 --------------
void ADC_Config(void){ //konfigurowanie przetwornika AC
  ADC_InitTypeDef ADC_InitStructure;
	  
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;                  // jeden przetwornik, praca niezalezna
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;                       // pomiar jednego kanalu, skanowanie kanalow nie potrzebne
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;               	// pomiar w trybie jednokrotnym
  //ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;                  // pomiar w trybie ciaglym
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;	// brak wyzwalania zewnetrznego
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;            	// wyrownanie danych do prawej - 12 mlodszych bitow znaczacych
	ADC_InitStructure.ADC_NbrOfChannel = 1; 	                          // liczba uzywanych kanalow =1
	ADC_Init(ADC1, &ADC_InitStructure);                                 // incjalizacja przetwornika

	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_1Cycles5); 
                                                                      // kanal 14 - GPIOC4 - potencjometr na plytce ZL27ARM
                                                                      // grupa podstawowa, czas probkowania 1,5 + 12,5 = 14 cykli
	ADC_Cmd(ADC1, ENABLE);                                              // wlacz ADC1

	ADC_ResetCalibration(ADC1);                                         // reset rejestrow kalibracyjnych ADC1
	while(ADC_GetResetCalibrationStatus(ADC1));                         // czekamy na wykonanie resetu
	ADC_StartCalibration(ADC1);                                         // kalibracja ADC1
	while(ADC_GetCalibrationStatus(ADC1));                              // czekamy na zakonczenie kalibracji ADC1
}
void Delay(unsigned int counter){
	// opoznienie programowe, 8'000'000 to jedna sekunda
	while (counter--){
		__NOP();
		__NOP();
	}
}
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/




