/******************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 	*
 *  STM3210E-EVAL:  PROJEKT_26 I2C ST-LM75 				  	*
 ******************************************************/
#include "stm32f10x.h"
#include "stm3210e_eval_lcd.h"
#include "stm3210e_eval_i2c_tsensor.h"
#include <stdio.h>
#include <stdlib.h>
#define LM75_addr 0x90         // 1001 000x

int  RCC_Config(void);
void GPIO_Config(void);
void Delay(unsigned int);

// zmienne globalne -------------------------------------------
unsigned char txt2LCD[32] = {0};// bufor funkcji sprintf
static uint8_t TempCelsius_Display[] = "     +abc.def C     ";
static uint8_t TempFahrenheit_Display[] = "     +abc.def F     ";
static uint32_t Temp_Decimal = 0, TempCelsius_Value = 0, Temp_Value_Fahrenheit = 0;

int main(void) {
	uint32_t i = 0;
  //konfiguracja systemu --------------------------------------
  RCC_Config();		
  GPIO_Config();
  GPIO_ResetBits(GPIOF, GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9);	// zgasimy wszystkie diody LED
  STM3210E_LCD_Init();		// inicjalizacja TFT LCD 240320L8
	LCD_Clear(Black);				// Clear the LCD
	LCD_SetFont(&Font16x24);
  LCD_SetBackColor(Black);		// kolor tla
	while(1){
		// Displays the temperature in Celsius and Fahrenheit degree.	
		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);
		LM75_Init();		// Initialize the Temperature Sensor 
		if(LM75_GetStatus() == SUCCESS){
			RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);
			LCD_SetTextColor(Green);   // kolor textu
			LCD_DisplayStringLine(Line0, (uint8_t *)"   I2C: STLM75M2E   ");
			LCD_DisplayStringLine(Line1, (uint8_t *)"czujnik temperatury ");
			
			RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);
			LCD_WindowModeDisable();			// Disable LCD Window mode

			LCD_SetTextColor(White);
			LCD_DisplayStringLine(Line4, (uint8_t *)"    Temperatura");
			LCD_SetTextColor(Blue);
			RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);
			
      LM75_Init();		// Initialize the Temperature Sensor
      TempCelsius_Value = LM75_ReadTemp() << 2;		// Get double of Temperature value
			RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);	
			
			if(TempCelsius_Value <= 1023){
        TempCelsius_Display[5] = '+';				// zmierzona temperatura jest dodatnia
        TempFahrenheit_Display[5] = '+'; 
      }
      else{
        TempCelsius_Display[5] = '-'; 			// zmierzona temperatura jest dodatnia
        TempFahrenheit_Display[5] = '-'; 
        TempCelsius_Value = 0x800 - TempCelsius_Value;	// usuwamy znak
      }
      /* Calculate temperature digits in C */
      Temp_Decimal = ((TempCelsius_Value & 7) * 1000 / 8);
      TempCelsius_Display[10] = (Temp_Decimal / 100) + 0x30;
      TempCelsius_Display[11] = ((Temp_Decimal % 100) / 10) + 0x30;
      TempCelsius_Display[12] = ((Temp_Decimal % 100) % 10) + 0x30;
      TempCelsius_Value >>= 3;
      TempCelsius_Display[6] = (TempCelsius_Value / 100) + 0x30;
      TempCelsius_Display[7] = ((TempCelsius_Value % 100) / 10) + 0x30;
      TempCelsius_Display[8] = ((TempCelsius_Value % 100) % 10) + 0x30;

      /* Convert temperature C to Fahrenheit */
      Temp_Value_Fahrenheit = ((9 * ((TempCelsius_Value * 1000) + Temp_Decimal)) / 5) + 32000;

      /* Calculate temperature digits in F */
      TempFahrenheit_Display[6] = (Temp_Value_Fahrenheit / 100000) + 0x30;
      TempFahrenheit_Display[7] = ((Temp_Value_Fahrenheit % 100000) /10000) + 0x30;
      TempFahrenheit_Display[8] = ((Temp_Value_Fahrenheit % 100000) %10000/1000) + 0x30;
      TempFahrenheit_Display[10] = ((((Temp_Value_Fahrenheit % 100000) %10000) %1000) /100) + 0x30;
      TempFahrenheit_Display[11] = (((((Temp_Value_Fahrenheit % 100000) %10000) %1000) %100) /10) + 0x30;
      TempFahrenheit_Display[12] = (((((Temp_Value_Fahrenheit % 100000) %10000) %1000) %100) %10) + 0x30;

      /* Display Fahrenheit value on LCD */
      for(i = 0; i < 20; i++){
        LCD_DisplayChar(Line6, (319 - (16 * i)), TempCelsius_Display[i]);
        LCD_DisplayChar(Line7, (319 - (16 * i)), TempFahrenheit_Display[i]);
      }
      RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, DISABLE);
    }
		Delay(1000000);
  }
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);// GPIO PB6 = SCL, PB7 = SDA
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE);// GPIO PF6/7/8/9 = LEDy
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 
	
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOF, &GPIO_InitStructure);
}
void Delay(unsigned int counter){
	// opoznienie programowe, 8'000'000 to jedna sekunda
	while (counter--){
		__NOP();
		__NOP();
	}
}
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/




